import os
import shutil

from si_tests import settings
from si_tests.deployments.utils import commons, file_utils, namespace
from si_tests.utils import utils
from . import base


class RefappTerraform(base.RefappBase):
    source = file_utils.join(commons.REFAPP_DIR, "terraform")

    def _exec_terraform(self, cmd_list):
        cmd = " && ".join(["./terraform " + c for c in cmd_list])
        return self.kubectl.exec(
            self.ostcm.client.name,
            f"bash -c 'cd /tmp/terraform; {cmd}'",
        ).result_str

    def _ensure_stuff_exists(self):
        try:
            self.kubectl.exec(self.ostcm.client.name, "[ -d /tmp/terraform ]")
        except AssertionError:
            # convert symlink to service library
            service_lib_link = f"{self.source}/templates/service_lib"
            if os.path.islink(service_lib_link):
                service_lib = os.path.realpath(service_lib_link)
                os.unlink(service_lib_link)
                shutil.copy(service_lib, service_lib_link)

            # copy terraform binary
            shutil.copy(utils.get_binary_path("terraform"), self.source)

            # store ssh keys
            key_file = f"{self.source}/templates/.openstack"
            if not all([os.path.exists(k) for k in (key_file, f"{key_file}.pub")]):
                key = utils.load_keyfile(settings.OPENSTACK_DEPLOY_SSH_KEY_PATH)
                utils.dump_keyfile(key_file, key)
                with open(f"{key_file}.pub", "w") as f:
                    f.write("ssh-rsa " + key["public"])

            commons.LOG.info("Upload terraform stuff to keystone-client pod")
            destination = f"{namespace.NAMESPACE.openstack}/{self.ostcm.client.name}:/tmp"
            self.kubectl.cp(self.source, destination)
            self._exec_terraform(["init"])

    @property
    def url(self):
        if self._url is None:
            self._url = self._exec_terraform(["output -raw app_url"])
        return self._url

    def deploy(self):
        self._ensure_stuff_exists()
        commons.LOG.info(
            "Create infrastructure according to Terraform configuration"
        )
        image = (
            f" -var='app_docker_image={settings.OPENSTACK_REFAPP_IMAGE}'"
            if settings.OPENSTACK_REFAPP_IMAGE
            else ""
        )
        self._exec_terraform(["apply -auto-approve" + image])
        # get current state back
        tfstate = "terraform.tfstate"
        destination = f"{namespace.NAMESPACE.openstack}/{self.ostcm.client.name}:/tmp"
        self.kubectl.cp(f"{destination}/terraform/{tfstate}", f"{self.source}/{tfstate}")
        commons.LOG.info("Refapp has been deployed successfully")

    def delete(self):
        self._ensure_stuff_exists()
        commons.LOG.info("Delete RefApp")
        self._exec_terraform(["destroy -auto-approve"])
        commons.LOG.info("Refapp has been deleted successfully")
