#    Copyright 2020 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os
import pytest

from si_tests import logger
from si_tests import settings
from si_tests.managers import bootstrap_manager
from si_tests.utils import waiters

LOG = logger.logger


class TestProvisionEquinixmetalv2RegionalClusters():

    @pytest.mark.usefixtures("introspect_standalone_seed_regional_deploy_objects")
    def test_deploy_equinixmetalv2_regional(self, kaas_manager, show_step):
        """Deploy Regional cluster on KaaS mgmt

        Scenario:
            1. Prepare standalone seed node
            2. Upload the management cluster kubeconfig into seed node user home
            3. Configure equinix network setup if deploy without Eqv2 mgmt
            4. Pass Region var and run bootsrap.sh deploy_regional
            5. Check result of deployment
        """

        assert waiters.icmp_ping(settings.SEED_STANDALONE_EXTERNAL_IP), (
            f"EM2 seed node for region cluster is not accessible by IP {settings.SEED_STANDALONE_EXTERNAL_IP}")
        bootstrap = bootstrap_manager.BootstrapManager(seed_ip=settings.SEED_STANDALONE_EXTERNAL_IP)

        show_step(1)
        bootstrap.create_seed_bash_completion()
        bootstrap.step_003_prepare_standalone_seed_node()

        show_step(2)
        # copy mgmt kubeconfig from default bootstrap user space
        if not settings.SI_BINARIES_DIR:
            raise Exception("WORKSPACE environment variable is not set, unable to find 'kubeconfig'")

        regional_cluster_bootstrap_userspace = settings.SEED_SSH_LOGIN
        regional_home = f"/home/{regional_cluster_bootstrap_userspace}"

        bootstrap.copy_user_file(
          os.path.join(settings.SI_BINARIES_DIR, "kubeconfig"),
          regional_cluster_bootstrap_userspace,
          regional_home + "/kubeconfig")

        show_step(3)
        if settings.KAAS_EQUINIX_PRIVATE_NETWORK_CONFIG:
            custom_region = True
            management_network_config, region_network_configs, child_network_configs = \
                bootstrap.equinixmetalv2_configure_networking(True)

            kaas_manager.si_config.create_si_scenario()
            kaas_manager.si_config.create_equinixmetalv2_network_config(
                management_network_config, region_network_configs, child_network_configs)
            extra_config_temp = kaas_manager.si_config.data
        else:
            extra_config_temp = kaas_manager.si_config.data
            custom_region = False

        show_step(4)
        bootstrap.step_004_deploy_kaas_regional(
            region=settings.EQUINIXMETALV2_REGION,
            extra_config=extra_config_temp,
            custom_region=custom_region)

        show_step(5)
        bootstrap.step_006_wait_for_pods()
        bootstrap.step_006_postdeployment_checks()

        ns = kaas_manager.get_namespace(settings.REGION_NAMESPACE)
        regional_cluster = ns.get_cluster(settings.REGIONAL_CLUSTER_NAME)
        regional_cluster.check.check_machines_status()
        regional_cluster.check.check_cluster_nodes(timeout=900)
        regional_cluster.check.check_helmbundles()
        regional_cluster.check.check_persistent_volumes_mounts()

        regional_cluster.store_k8s_artifacts()
        regional_cluster.store_machines_artifacts(public_ip=False)

        # Check/wait for correct docker service replicas in cluster
        regional_cluster.check.check_actual_expected_docker_services()
        regional_cluster.check.check_k8s_pods()
        regional_cluster.check.check_cluster_readiness()
        regional_cluster.check.check_actual_expected_pods(timeout=1200)
        regional_cluster.provider_resources.save_artifact()
